searchData={"items":[{"type":"module","title":"LiveAttribute","doc":"LiveAttribute makes binding updateable values easier. To use it add it to your LiveView using `use LiveAttribute`\nand then use the function `assign_attribute(socket, subscribe_callback, property_callbacks)` to register attributes.\n\nThe attributes will listen to all incoming events and update their assigns of your LiveView automatically, saving\nyou the hassle of implementing independent `handle_info()` and `update_...()` calls.","ref":"LiveAttribute.html"},{"type":"module","title":"Example using LiveAttribute - LiveAttribute","doc":"```\ndefmodule UserLive do\n  use Phoenix.LiveView\n  use LiveAttribute\n\n  def mount(_params, _session, socket) do\n    {:ok, assign_attribute(socket, &User.subscribe/0, users: &User.list_users/0)}\n  end\n\n  def handle_event(\"delete_user\", %{\"id\" => user_id}, socket) do\n    User.get_user!(user_id)\n    |> User.delete_user()\n\n    {:noreply, socket}\n  end\nend\n```","ref":"LiveAttribute.html#module-example-using-liveattribute"},{"type":"module","title":"Same Example without LiveAttribute - LiveAttribute","doc":"```\ndefmodule UserLive do\n  use Phoenix.LiveView\n\n  def mount(_params, _session, socket) do\n    if connected?(socket), do: User.subscribe()\n    {:ok, update_users(socket)}\n  end\n\n  defp update_users(socket) do\n    users = User.list_users()\n    assign(socket, users: users)\n  end\n\n  def handle_event(\"delete_user\", %{\"id\" => user_id}, socket) do\n    User.get_user!(user_id)\n    |> User.delete_user()\n\n    {:noreply, socket}\n  end\n\n  def handle_info({User, [:user, _], _}, socket) do\n    {:noreply, update_users(socket)}\n  end\nend\n```","ref":"LiveAttribute.html#module-same-example-without-liveattribute"},{"type":"module","title":"assign\\_attribute(socket, subscribe, filter \\\\ :\\_, refresher) - LiveAttribute","doc":"* `socket` the LiveView socket where the assigns should be executed on\n* `subscribe` the subscribe callback to start the subscription e.g. `&Users.subscribe/0`\n* `filter` an optional filter if you don't want to update on each event. The filter can either be an expression\n  using `:_` as wildcard parameter such as `{User, [:user, :_], :_}`. Alternatively `filter`\n  can be a function with one parameter\n  _Note_ LiveAttribute is issuing each subscribe call in an isolated helper process, so you only need\n  to add filters to reduce the scope of a single subscription.\n* `refresher` the function callback to load the new values after a subscription event has\n  fired.","ref":"LiveAttribute.html#module-assign-_attribute-socket-subscribe-filter-_-refresher"},{"type":"function","title":"LiveAttribute.assign_attribute/2","doc":"Shortcut version of `assign_attribute` to capture an attribute configuration\nin a tuple and re-use in multiple LiveViews. This accepts two-element and three-\nelement tuples with: `{subscribe, refresher}` or  `{subscribe, filter, refresher}`\ncorrespondingly\n\nUse with:\n```\nsocket = assign_attribute(socket, User.live_attribute())\n```\n\nWhen there is an `User` method:\n\n```\ndefmodule User do\n  def live_attribute() do\n    {&subscribe/0, users: &list_users/0}\n  end\n  ...\nend\n```","ref":"LiveAttribute.html#assign_attribute/2"},{"type":"function","title":"LiveAttribute.assign_attribute/4","doc":"```\nsocket = assign_attribute(socket, &User.subscribe/0, users: &User.list/0)\n```\n\n`assign_attribute` updates the specified assign keys each time there is a new event sent from\nthe subscription source.\n\nSee `refresher()` and `filter()` for advanced usage of these parameters. Simple usage:","ref":"LiveAttribute.html#assign_attribute/4"},{"type":"function","title":"LiveAttribute.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"LiveAttribute.html#child_spec/1"},{"type":"type","title":"LiveAttribute.filter/0","doc":"The filter allows doing optimzation by a) ignoring certain events of the subscription\nsource and b) pass event values directly to assign values, instead of using refresher\nfunctions to re-load them.\n\nIt can be either a match object defining which events should be matched, or a function\nreturning `false` when the event should be ignored or a map when it should be processed.\nKeys that are present in the map will be assigned to the socket. (if there are matching\nkeys in the refresher list)","ref":"LiveAttribute.html#t:filter/0"},{"type":"type","title":"Filter function - LiveAttribute.filter/0","doc":"The filter function receives the event and should return either `false` or a map of\nthe new values:\n\n```\nfn event ->\n  case event do\n    {User, :users_updated, users} -> %{users: users}\n    _ -> false\n  end\nend)\n```","ref":"LiveAttribute.html#t:filter/0-filter-function"},{"type":"type","title":"Filter object - LiveAttribute.filter/0","doc":"Match objects are defined by example of a matching list or tuple. These can be customized\nusing two special terms:\n- `:_` the wildcard which matches any value, but ignores it\n- `{:\"$\", some_key}` - which matches any value, and uses it as update value in the socket assigns","ref":"LiveAttribute.html#t:filter/0-filter-object"},{"type":"type","title":"Examples - LiveAttribute.filter/0","doc":"```\n# Let's assumg the `User` module is generating the following event each time\n# the user list is updated: `{User, :users_updated, all_users}`\n# then the following match object will extract the users\n\n{User, :users_updated, {:\"$\", :users}}\n\n# Full function call with match object\nassign_attribute(socket, &User.subscribe/0, users: &User.list/0, {User, :users_updated, {:\"$\", :users}})\n\n\n# Now the same we could get with this function callback instead:\nfn event ->\n  case event do\n    {User, :users_updated, users} -> %{users: users}\n    _ -> false\n  end\nend)\n\n# Full function call with callback\nassign_attribute(socket, &User.subscribe/0, users: &User.list/0,\n  fn event ->\n    case event do\n      {User, :users_updated, users} -> %{users: users}\n      _ -> false\n    end\n  end)\n```","ref":"LiveAttribute.html#t:filter/0-examples"},{"type":"type","title":"LiveAttribute.refresher/0","doc":"The refresher list is passed to `assign_attribute()` to know which assigns to update when\nthe subscription source issues an event.\n\nIt is a list of `{key, callback}` pairs specifying how to load the new attribute values.\nThe `callback` thereby can have optionally one argument to read context from the socket.","ref":"LiveAttribute.html#t:refresher/0"},{"type":"type","title":"refresher() examples - LiveAttribute.refresher/0","doc":"```\n# 1. Zero argument callback to update the users list:\n[users: &User.list_all/0]\n\n# 2. Single argument callback to use the socket state in the update:\n[users: fn socket ->\n  User.list_all() -- socket.assigns.blacklist\nend]\n\n# 3. Special `socket` key to assign multiple values at once manually\n[socket: fn socket ->\n  assign(socket,\n    users: User.list_all() -- socket.assigns.blacklist,\n    last_update: System.os_time()\n  )\nend]\n```","ref":"LiveAttribute.html#t:refresher/0-refresher-examples"},{"type":"type","title":"Usage Examples - LiveAttribute.refresher/0","doc":"```\niex> assign_attribute(socket, &User.subscribe(), users: &User.list_all/0)\n\niex> assign_attribute(socket, &User.subscribe(), fn socket ->\n  assign(socket, users: User.list_all() -- socket.assigns.blacklist)\nend)\n```","ref":"LiveAttribute.html#t:refresher/0-usage-examples"},{"type":"type","title":"LiveAttribute.socket/0","doc":"","ref":"LiveAttribute.html#t:socket/0"},{"type":"function","title":"LiveAttribute.update_attribute/2","doc":"```\n    socket = update_attribute(socket, :users)\n  ```\n\n  Helper method to issue a update callback manually on a live attribute, when there\n  is a known update but no subscription event.","ref":"LiveAttribute.html#update_attribute/2"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.1"}}